#ifndef Redukce_h
#define Redukce_h
#include <BK/Struct.h>
#include <cmath>

// zkresleni do JTSK z delky pruvodice
double ro2m( const double& ro )
{ double dR = ro - 1298039;
  return 0.9999 + 1e-14 *dR*dR *(1.22822 - dR * 1e-7 * (3.154 - dR * 1e-6 * (1.848 - dR * 1e-6 * 1.15)));
}

// zkresleni do JTSK ze souradnic bodu
double mJTSK( Bod P, Bod K )
{ Bod M;
  M.x = (P.x+K.x)/2;
  M.y = (P.y+K.y)/2;
  double mP = ro2m( sqrt(P.x*P.x+P.y*P.y) );
  double mK = ro2m( sqrt(K.x*K.x+K.y*K.y) );
  double mM = ro2m( sqrt(M.x*M.x+M.y*M.y) );
  if( P.x==1e999 | P.y==1e999 ) mP = mM = mK;
  if( K.x==1e999 | K.y==1e999 ) mK = mM = mP;
  if( (P.x==1e999 | P.y==1e999) & (K.x==1e999 | K.y==1e999) )
    throw Chyba("Nemohu redukovat do JTSK, nejsou-li zadny souadnice bodu/bod.");
  return (mP + 4*mM + mK) / 6;
} 

// redukce ...
#define D0     1  // do nuloveho horizontu,
#define KK     2  // na spojnici stabilizacnich znacek
#define JTSK   4  // do zobrazovaci roviny S-JTSK
#define VYSKY  8  // Redukce pres vysky
Mereni Redukce( Mereni mP, Mereni mK, Bod P, Bod K, int flags=JTSK, const double k=0.13, const double R=6380703.6105 )
{ 
  // mereni pro ukadani vysledku
  Mereni rmP;
  rmP.Hz = mP.Hz;
  // prevod na radiany (pred navratem prevod zpet)
  mP.Z = mP.Z*M_PI/200;
  mK.Z = mK.Z*M_PI/200;
  // zvyseni o vysku pristroje a cile
  if( mP.vp!=1e999 & P.z!=1e999 ) P.z += mP.vp;
  if( mP.vc!=1e999 & K.z!=1e999 ) K.z += mP.vc;
    
  // NULOVY HORIZONT (D0)
  // Znamy presne nadmorske vysky. Redukce bez zenitoveho uhlu.
  if( Flags( flags,VYSKY ) )
    if( P.z!=1e999 & K.z!=1e999 ) 
    { rmP.D = sqrt(   ( mP.D*mP.D-(K.z-P.z)*(K.z-P.z) )   /(1+P.z/R)/(1+K.z/R)    );
      rmP.tD = '0';    MessageBox(0,String(mJTSK( P,K )).c_str(),0,0);
      if( Flags( flags,JTSK ) ) { rmP.D = rmP.D * mJTSK( P,K ); return rmP; }
      else return rmP;
    }
    else throw Chyba("Nemohu redukovat pouze pomoc vek koncovch bod, nejsou-li znmy.");
  
  // Stredni nadmorska vyska
  double Hm = 0;
  if( P.z!=1e999 & K.z!=1e999 ) Hm = (P.z+K.z)/2;
  else if( P.z!=1e999 & mP.Z!=1e999 ) Hm = (2*P.z+mP.D*cos(mP.Z))/2;
  else if( P.z!=1e999 & mK.Z!=1e999 ) Hm = (2*P.z+mP.D*cos(M_PI-mK.Z))/2;
  else if( K.z!=1e999 & mK.Z!=1e999 ) Hm = (2*K.z+mP.D*cos(mK.Z))/2;
  else if( K.z!=1e999 & mP.Z!=1e999 ) Hm = (2*K.z+mP.D*cos(M_PI-mP.Z))/2;
  else throw Chyba("Nemohu redukovat, nen-li znm ani jeden zenitov hel nebo ani jedna vka bodu.");
  
  // geocentricky uhel
  double fi;
  if( mP.Z!=1e999 ) fi = mP.D*sin(mP.Z)/(R+Hm);
  else if( mK.Z!=1e999 ) fi = mP.D*sin(M_PI-mK.Z)/(R+Hm);
  else throw Chyba("Nemohu redukovat, nen-li znm ani jeden zenitov hel.");

  // refrakce: refrakcni uhel, red. delky
  double ro;
  if( mP.Z!=1e999 & mK.Z!=1e999 & mP.vp==mK.vc & mK.vp==mP.vc )
    ro = ( M_PI+fi- mP.Z-mK.Z )/2;
  else ro = k*fi/2;
  mP.D = mP.D - (2*k-k*k) * (mP.D*mP.D*mP.D) /24 /(R*R);

  // pro moznost vypoctu pri mereni zenitoveho uhlu pouze na koncovem bode
  if( mP.Z==1e999 ) mP.Z = M_PI+fi-mK.Z;

  // SPOJNICE STABILIZACNICH ZNACEK (KK)
  if( Flags( flags,KK ) )
  { if( mP.vp==1e999 | mP.vc==1e999 )
      throw Chyba("Nemohu redukovat na spojnici stabilizanch znaek,\nnejsou-li meny vka pstroje a cle.");
    double dv = mP.vc-mP.vp;
    double alfa = mP.Z +ro -fi;               // uhel pri bode 'K'
    // spojnice rovnobezna se spojnici stabilizacnich znacek
    double DkkII = sqrt(  mP.D*mP.D +dv*dv -2*mP.D*dv*cos(alfa)  );
           rmP.D = DkkII - fi*dv;             // spojnice kamen - kamen
    double dZ = asin( sin(alfa)*dv/DkkII );   // redukce Z
           rmP.Z = mP.Z + ro + dZ;            // redukovany Z z bodu P
           rmP.Z = rmP.Z*200/M_PI;
           rmP.tD = '2'; // symbol pro redukci na kamen - kamen
    return rmP;
  }
  
  // NULOVY HORIZONT (D0)
  // redukce primo pres vodorovnou ve stredni nadmorske vysce
  if( Flags( flags,D0 ) | Flags( flags,JTSK ) )
  { double Dm = mP.D*sin( mP.Z+ro-fi/2 );     // HD ve sredni nadm.H
           rmP.D = Dm*R /(R+Hm);              // D v nulovem horizontu
           rmP.tD = '0';
    if( flags==JTSK )
    { rmP.D = rmP.D * mJTSK( P,K );  
      return rmP; }
    else return rmP;
  }
}
#endif
